@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/remote.c,v 1.7 2009/09/28 22:39:48 william Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/remote.c,v 1.9 2011/05/16 16:21:59 william Exp $
  */
 
 #define _INCLUDE_FROM_REMOTE_C_
@@ -539,6 +539,7 @@ char *WriteRemoteFileIntoTemp(psz_buf, buf_sz, psz_ext)
 {
    char *tmp_fname=NULL;
    int fd=(-1);
+   mode_t mode=((tmpFileMode == 0) ? 0700 : tmpFileMode);
 
    tmp_fname = (char*)malloc((strlen(tmpDir)+20)*sizeof(char));
    if (tmp_fname == NULL) {
@@ -552,7 +553,7 @@ char *WriteRemoteFileIntoTemp(psz_buf, buf_sz, psz_ext)
    }
    if (psz_ext != NULL) strcat(tmp_fname, psz_ext);
    unlink(tmp_fname);
-   if ((fd=open(tmp_fname, O_WRONLY|O_CREAT|O_TRUNC, tmpFileMode)) == (-1)) {
+   if ((fd=open(tmp_fname, O_WRONLY|O_CREAT|O_TRUNC, mode)) == (-1)) {
       sprintf(gszMsgBox, TgLoadString(STID_CANT_CREATE_NAMED_TMP_FILE),
             tmp_fname);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
