@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgisdl.c,v 1.13 2009/10/15 03:38:40 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/tgisdl.c,v 1.15 2011/05/16 16:22:00 william Exp $
  */
 
 #define _INCLUDE_FROM_TGISDL_C_
@@ -973,6 +973,9 @@ int TdgtInsertSymbolDlgLoop(pti)
          }
       } else if (xev.type == Expose) {
          ExposeEventHandler(&xev, FALSE);
+      } else if (xev.type == ConfigureNotify &&
+            xev.xany.window==mainWindow) {
+         Reconfigure(FALSE);
       } else if (xev.type == KeyPress) {
          TidgetManagerHandleAllKeyPressEvent(&xev);
       } else if (IsTdgtNotifyEvent(pti, &xev, &tdgt_notify)) {
